Struct Obligation
pub struct Obligation<'db, T> {
pub cause: ObligationCause,
pub param_env: ParamEnv<'db>,
pub predicate: T,
pub recursion_depth: usize,
}Expand description
An Obligation represents some trait reference (e.g., i32: Eq) for
which the “impl_source” must be found. The process of finding an “impl_source” is
called “resolving” the Obligation. This process consists of
either identifying an impl (e.g., impl Eq for i32) that
satisfies the obligation, or else finding a bound that is in
scope. The eventual result is usually a Selection (defined below).
Fields
cause: ObligationCauseThe reason we have to prove this thing.
param_env: ParamEnv<'db>The environment in which we should prove this thing.
predicate: TThe thing we are trying to prove.
recursion_depth: usizeIf we started proving this as a result of trying to prove something else, track the total depth to ensure termination. If this goes over a certain threshold, we abort compilation – in such cases, we can not say whether or not the predicate holds for certain. Stupid halting problem; such a drag.
Implementations
impl<'db, T> Obligation<'db, T>where
T: Copy,
impl<'db, T> Obligation<'db, T>where
T: Copy,
pub fn as_goal(&self) -> Goal<DbInterner<'db>, T>
impl<'db> Obligation<'db, Predicate<'db>>
impl<'db> Obligation<'db, Predicate<'db>>
pub fn flip_polarity(
&self,
_interner: DbInterner<'db>,
) -> Option<Obligation<'db, Predicate<'db>>>
pub fn flip_polarity( &self, _interner: DbInterner<'db>, ) -> Option<Obligation<'db, Predicate<'db>>>
Flips the polarity of the inner predicate.
Given T: Trait predicate it returns T: !Trait and given T: !Trait returns T: Trait.
impl<'db, O> Obligation<'db, O>
impl<'db, O> Obligation<'db, O>
pub fn new( interner: DbInterner<'db>, cause: ObligationCause, param_env: ParamEnv<'db>, predicate: impl Upcast<DbInterner<'db>, O>, ) -> Obligation<'db, O>
pub fn set_depth_from_parent(&mut self, parent_depth: usize)
pub fn set_depth_from_parent(&mut self, parent_depth: usize)
We often create nested obligations without setting the correct depth.
To deal with this evaluate and fulfill explicitly update the depth of nested obligations using this function.
pub fn with_depth( interner: DbInterner<'db>, cause: ObligationCause, recursion_depth: usize, param_env: ParamEnv<'db>, predicate: impl Upcast<DbInterner<'db>, O>, ) -> Obligation<'db, O>
pub fn with<P>( &self, tcx: DbInterner<'db>, value: impl Upcast<DbInterner<'db>, P>, ) -> Obligation<'db, P>